package com.perforce.workshop.tjuricek.p4java_ext;

import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Access to the "p4 files" command
 */
public interface P4Files extends P4Connection, P4Errors {

    static P4Files create(P4Connection conn) {
        return new DefaultP4Files(conn);
    }

    /**
     * Returns the result of "p4 files //path...@=id"
     *
     * @param path     If non-null, will use this path selector to trim down the path, otherwise, will
     *                 use "//..." with the revision specifier
     * @param changeId The specific change ID that eventually is used to specify the selected
     *                 path in "//...@=ID" syntax
     * @return The list of FileSummary objects
     */
    default List<FileSummary> listChangelist(String path, String changeId) {
        try {
            if (path == null) {
                path = "//...";
            }

            String[] opts = new String[]{path + "@=" + changeId};

            IOptionsServer server = getOptionsServer();

            Map<String, Object>[] results = server.execMapCmd("files", opts, null);

            checkError(results);

            List<FileSummary> summaries = new ArrayList<FileSummary>();

            for (Map<String, Object> map : results) {
                summaries.add(new FileSummary(map));
            }

            return summaries;

        } catch (P4JavaException p4) {
            throw new P4Exception(p4);
        }
    }

    /**
     * Returns the result of <code>p4 files [path1 path2 ...]</code>.
     *
     * @param paths Should be at least one path to list, or you'll likely get
     *              an exception
     * @return FileSummary objects for each line of output
     */
    default List<FileSummary> list(String... paths) {
        try {
            Map<String, Object>[] results =
                    getOptionsServer().execMapCmd("files", paths, null);

            checkError(results);

            return Arrays.asList(results).stream()
                    .map(m -> new FileSummary(m))
                    .collect(Collectors.toList());

        } catch (P4JavaException p4) {
            throw new P4Exception(p4);
        }
    }
}
